@@ -27,13 +27,22 @@
  * -------------------------------------------------------------------------- */
 
 #include "config.h" /* for HAVE_LIBUSB_DETACH_KERNEL_DRIVER flag */
+
+#if (defined ENABLE_SHARED_PRIVATE_LIBS) && ENABLE_SHARED_PRIVATE_LIBS
+# if (defined BUILD_FOR_SHARED_PRIVATE_LIBS) && BUILD_FOR_SHARED_PRIVATE_LIBS
+#  define suggest_NDE_conflict          LIBNUTPRIVATE_suggest_NDE_conflict
+/* else: would need to pass method pointer like in main.c, too much
+ * hassle for a mixed-dynamicity build that might never happen */
+# endif
+#endif
+
 #include "common.h" /* for xmalloc, upsdebugx prototypes */
 #include "usb-common.h"
 #include "nut_libusb.h"
 #include "nut_stdint.h"
 
 #define USB_DRIVER_NAME		"USB communication driver (libusb 1.0)"
-#define USB_DRIVER_VERSION	"0.50"
+#define USB_DRIVER_VERSION	"0.53"
 
 /* driver description structure */
 upsdrv_info_t comm_upsdrv_info = {
@@ -185,7 +194,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 #endif
 	/* libusb-1.0 usb_ctrl_charbufsize is uint16_t and we
 	 * want the rdlen vars signed - so taking a wider type */
-	int32_t rdlen1, rdlen2; /* report descriptor length, method 1+2 */
+	int32_t rdlen1, rdlen2, rdlens[2]; /* report descriptor length, method 1+2, then an array to iterate them (if both) in chosen order */
 	USBDeviceMatcher_t *m;
 	libusb_device **devlist;
 	ssize_t	devcount = 0;
@@ -203,13 +212,14 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 	const unsigned char *p;
 	char string[256];
 	int i;
+	size_t j;
 	int count_open_EACCESS = 0;
 	int count_open_errors = 0;
 	int count_open_attempts = 0;
 
 	/* report descriptor */
 	unsigned char	rdbuf[MAX_REPORT_SIZE];
-	int32_t		rdlen;
+	int32_t		rdlen = -1;
 
 	static int	usb_hid_number_opts_parsed = 0;
 	if (!usb_hid_number_opts_parsed) {
@@ -358,7 +368,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 			libusb_free_device_list(devlist, 1);
 			fatal_with_errno(EXIT_FAILURE, "Out of memory");
 		}
-		sprintf(curDevice->Bus, "%03d", bus_num);
+		snprintf(curDevice->Bus, 4, "%03d", bus_num);
 
 		device_addr = libusb_get_device_address(device);
 		curDevice->Device = (char *)malloc(4);
@@ -368,7 +378,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 		}
 		if (device_addr > 0) {
 			/* 0 means not available, e.g. lack of platform support */
-			sprintf(curDevice->Device, "%03d", device_addr);
+			snprintf(curDevice->Device, 4, "%03d", device_addr);
 		} else {
 			if (devnum <= 999) {
 				/* Log visibly so users know their number discovered
@@ -376,7 +386,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 				upsdebugx(0, "%s: invalid libusb device address %" PRIu8 ", "
 					"falling back to enumeration order counter %" PRIuSIZE,
 					__func__, device_addr, devnum);
-				sprintf(curDevice->Device, "%03d", (int)devnum);
+				snprintf(curDevice->Device, 4, "%03d", (int)devnum);
 			} else {
 				upsdebugx(1, "%s: invalid libusb device address %" PRIu8,
 					__func__, device_addr);
@@ -393,7 +403,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 			fatal_with_errno(EXIT_FAILURE, "Out of memory");
 		}
 		if (bus_port > 0) {
-			sprintf(curDevice->BusPort, "%03d", bus_port);
+			snprintf(curDevice->BusPort, 4, "%03d", bus_port);
 		} else {
 			upsdebugx(1, "%s: invalid libusb bus number %i",
 				__func__, bus_port);
@@ -653,7 +663,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 		if (rdlen1 < -1) {
 			upsdebugx(2, "Warning: HID descriptor, method 1 failed");
 		}
-		upsdebugx(3, "HID descriptor length (method 1) %d", rdlen1);
+		upsdebugx(3, "HID descriptor length (method 1) %" PRIi32, rdlen1);
 
 		/* SECOND METHOD: find HID descriptor among "extra" bytes of
 		   interface descriptor, i.e., bytes tucked onto the end of
@@ -683,7 +693,7 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 		if (rdlen2 < -1) {
 			upsdebugx(2, "Warning: HID descriptor, method 2 failed");
 		}
-		upsdebugx(3, "HID descriptor length (method 2) %d", rdlen2);
+		upsdebugx(3, "HID descriptor length (method 2) %" PRIi32, rdlen2);
 
 		/* when available, always choose the second value, as it
 			seems to be more reliable (it is the one reported e.g. by
@@ -691,28 +701,40 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 			the maximum of the two values instead. */
 		if ((curDevice->VendorID == 0x463) && (curDevice->bcdDevice == 0x0202)) {
 			upsdebugx(1, "Eaton device v2.02. Using full report descriptor");
-			rdlen = rdlen1;
+			rdlens[0] = rdlen1;
+			rdlens[1] = rdlen2;
 		}
 		else {
-			rdlen = rdlen2 >= 0 ? rdlen2 : rdlen1;
+			rdlens[0] = rdlen2 >= 0 ? rdlen2 : rdlen1;
+			rdlens[1] = rdlen2 >= 0 ? rdlen1 : rdlen2;
 		}
 
-		if (rdlen < 0) {
+		if (rdlen1 < 0 && rdlen2 < 0) {
 			upsdebugx(2, "Unable to retrieve any HID descriptor");
 			goto next_device;
 		}
 		if (rdlen1 >= 0 && rdlen2 >= 0 && rdlen1 != rdlen2) {
 			upsdebugx(2, "Warning: two different HID descriptors retrieved "
-				"(Reportlen = %d vs. %d)", rdlen1, rdlen2);
+				"(Reportlen = %" PRIi32 " vs. %" PRIi32 ")",
+				rdlen1, rdlen2);
+		} else {
+			if (rdlen1 == rdlen2) {
+				rdlens[1] = -1;
+			}
 		}
 
-		upsdebugx(2, "HID descriptor length %d", rdlen);
+		for (j = 0; j < sizeof(rdlens); j++) {
+			rdlen = rdlens[j];
+			if (rdlen < 0)
+				continue;
 
-		if (rdlen > (int)sizeof(rdbuf)) {
-			upsdebugx(2, "HID descriptor too long %d (max %d)",
-				rdlen, (int)sizeof(rdbuf));
-			goto next_device;
-		}
+			upsdebugx(2, "Trying HID descriptor length %" PRIi32, rdlen);
+
+			if (rdlen > (int)sizeof(rdbuf)) {
+				upsdebugx(2, "HID descriptor too long %" PRIi32 " (max %d)",
+					rdlen, (int)sizeof(rdbuf));
+				continue;
+			}
 
 #if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
 # pragma GCC diagnostic push
@@ -726,67 +748,78 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 #ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE
 # pragma GCC diagnostic ignored "-Wtautological-unsigned-zero-compare"
 #endif
-		if ((uintmax_t)rdlen > UINT16_MAX) {
+			if ((uintmax_t)rdlen > UINT16_MAX) {
 #if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_UNSIGNED_ZERO_COMPARE) )
 # pragma GCC diagnostic pop
 #endif
-			upsdebugx(2, "HID descriptor too long %d (max %" PRIuMAX ")",
-				rdlen, (uintmax_t)UINT16_MAX);
-			goto next_device;
-		}
+				upsdebugx(2, "HID descriptor too long %" PRIi32
+					" (max %" PRIuMAX ")",
+					rdlen, (uintmax_t)UINT16_MAX);
+				continue;
+			}
 
-		/* libusb0: USB_ENDPOINT_IN + 1 */
-		res = libusb_control_transfer(udev,
-			LIBUSB_ENDPOINT_IN|LIBUSB_REQUEST_TYPE_STANDARD|LIBUSB_RECIPIENT_INTERFACE,
-			LIBUSB_REQUEST_GET_DESCRIPTOR,
-			(LIBUSB_DT_REPORT << 8) + usb_subdriver.hid_desc_index,
-			usb_subdriver.hid_rep_index,
-			rdbuf, (uint16_t)rdlen, USB_TIMEOUT);
+			/* libusb0: USB_ENDPOINT_IN + 1 */
+			res = libusb_control_transfer(udev,
+				LIBUSB_ENDPOINT_IN|LIBUSB_REQUEST_TYPE_STANDARD|LIBUSB_RECIPIENT_INTERFACE,
+				LIBUSB_REQUEST_GET_DESCRIPTOR,
+				(LIBUSB_DT_REPORT << 8) + usb_subdriver.hid_desc_index,
+				usb_subdriver.hid_rep_index,
+				rdbuf, (uint16_t)rdlen, USB_TIMEOUT);
 
-		if (res < 0)
-		{
-			upsdebug_with_errno(2, "Unable to get Report descriptor");
-			goto next_device;
-		}
+			if (res < 0)
+			{
+				upsdebug_with_errno(2, "Unable to get Report descriptor");
+				continue;
+			}
 
-		if (res < rdlen)
-		{
+			if (res < rdlen)
+			{
 #ifndef WIN32
-			upsdebugx(2, "Warning: report descriptor too short "
-				"(expected %d, got %d)", rdlen, res);
+				upsdebugx(2, "Warning: report descriptor too short "
+					"(expected %" PRIi32 ", got %d)", rdlen, res);
 #else	/* WIN32 */
-			/* https://github.com/networkupstools/nut/issues/1690#issuecomment-1455206002 */
-			upsdebugx(0, "Warning: report descriptor too short "
-				"(expected %d, got %d)", rdlen, res);
-			upsdebugx(0, "Please check your Windows Device Manager: "
-				"perhaps the UPS was recognized by default OS\n"
-				"driver such as HID UPS Battery (hidbatt.sys, "
-				"hidusb.sys or similar). It could have been\n"
-				"\"restored\" by Windows Update. You can try "
-				"https://zadig.akeo.ie/ to handle it with\n"
-				"either WinUSB, libusb0.sys or libusbK.sys.");
+				/* https://github.com/networkupstools/nut/issues/1690#issuecomment-1455206002 */
+				upsdebugx(0, "Warning: report descriptor too short "
+					"(expected %" PRIi32 ", got %d)", rdlen, res);
+				upsdebugx(0, "Please check your Windows Device Manager: "
+					"perhaps the UPS was recognized by default OS\n"
+					"driver such as HID UPS Battery (hidbatt.sys, "
+					"hidusb.sys or similar). It could have been\n"
+					"\"restored\" by Windows Update. You can try "
+					"https://zadig.akeo.ie/ to handle it with\n"
+					"either WinUSB, libusb0.sys or libusbK.sys.");
 #endif	/* WIN32 */
-			rdlen = res; /* correct rdlen if necessary */
+				rdlen = res; /* correct rdlen if necessary */
+			}
+
+			if (rdlen < USB_CTRL_CHARBUFSIZE_MIN
+			||  (uintmax_t)rdlen > (uintmax_t)USB_CTRL_CHARBUFSIZE_MAX
+			) {
+				upsdebugx(2,
+					"Report descriptor length is out of range on this device: "
+					"should be %" PRIdMAX " < %" PRIi32 " < %" PRIuMAX,
+						(intmax_t)USB_CTRL_CHARBUFSIZE_MIN, rdlen,
+						(uintmax_t)USB_CTRL_CHARBUFSIZE_MAX);
+				goto next_device;
+			}
+
+			res = callback(udev, curDevice, rdbuf, (usb_ctrl_charbufsize)rdlen);
+			if (res < 1) {
+				upsdebugx(2, "Caller doesn't like this device");
+				continue;
+			}
+
+			/* We found it... or at least something that did not complain */
+			break;
 		}
 
-		if (rdlen < USB_CTRL_CHARBUFSIZE_MIN
-		||  (uintmax_t)rdlen > (uintmax_t)USB_CTRL_CHARBUFSIZE_MAX
-		) {
-			upsdebugx(2,
-				"Report descriptor length is out of range on this device: "
-				"should be %" PRIdMAX " < %d < %" PRIuMAX,
-					(intmax_t)USB_CTRL_CHARBUFSIZE_MIN, rdlen,
-					(uintmax_t)USB_CTRL_CHARBUFSIZE_MAX);
+		if (j >= sizeof(rdlens)) {
+			/* Ended the loop without success */
 			goto next_device;
 		}
 
-		res = callback(udev, curDevice, rdbuf, (usb_ctrl_charbufsize)rdlen);
-		if (res < 1) {
-			upsdebugx(2, "Caller doesn't like this device");
-			goto next_device;
-		}
-
-		upsdebugx(2, "Report descriptor retrieved (Reportlen = %d)", rdlen);
+		upsdebugx(2, "Report descriptor retrieved (Reportlen = %" PRIi32 ")",
+			rdlen);
 		upsdebugx(2, "Found HID device");
 
 		upsdebugx(3, "Using default, detected or customized USB HID numbers: "
@@ -840,6 +873,12 @@ static int nut_libusb_open(libusb_device_handle **udevp,
 				"requested criteria",
 				count_open_attempts - count_open_errors);
 		}
+		suggest_NDE_conflict();
+		/* FIXME: Wiki names are inherently volatile;
+		 * maybe keep a more persistent copy in the
+		 * NUT-website with some knowledge-base ID? */
+		upsdebugx(1, "For more ideas, please check a NUT GitHub Wiki page named like "
+			"https://github.com/networkupstools/nut/wiki/%%22Insufficient-permissions%%22-when-starting-USB-drivers");
 	}
 
 #ifdef WIN32
