@@ -28,7 +28,7 @@
 #include "nut_stdint.h"
 
 #define DRIVER_NAME	"Metasystem UPS driver"
-#define DRIVER_VERSION	"0.13"
+#define DRIVER_VERSION	"0.14"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -61,6 +61,7 @@ static int nominal_power = 0;
 #define UPS_SET_BATTERY_TEST		0x0e
 
 static int instcmd(const char *cmdname, const char *extra);
+static void send_zeros(void);
 
 /*
 	Metasystem UPS data transfer are made with packet of the format:
@@ -113,7 +114,6 @@ static void send_zeros(void) {				/* send 100 times the value 0x00.....it seems
 
 	memset(buf, '\0', sizeof(buf));
 	ser_send_buf(upsfd, buf, sizeof(buf));
-	return;
 }
 
 
@@ -289,6 +289,9 @@ void upsdrv_initinfo(void)
 	char serial[13];
 	int res, i;
 
+	/* Reset comms */
+	send_zeros();
+
 	/* Initial setup of variables */
 #ifdef EXTRADATA
 	 dstate_setinfo("output.power", "%d", -1);
@@ -612,8 +615,8 @@ void upsdrv_initinfo(void)
 	dstate_addcmd("beeper.mute");
 	dstate_addcmd("beeper.on");
 	dstate_addcmd("beeper.off");
+
 	upsh.instcmd = instcmd;
-	return;
 }
 #if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_BESIDEFUNC) && (!defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP_INSIDEFUNC) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS_BESIDEFUNC) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE_BESIDEFUNC) )
 # pragma GCC diagnostic pop
@@ -1122,7 +1125,6 @@ void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B2400);
-	send_zeros();
 }
 
 void upsdrv_cleanup(void)
