@@ -26,7 +26,7 @@
 #include <stddef.h>
 #include "nut_stdint.h"
 
-#define MASTERGUARD_VERSION "Masterguard 0.04"
+#define MASTERGUARD_VERSION "Masterguard 0.05"
 
 /* series (for un-SKIP) */
 static char masterguard_my_series = '?';
@@ -328,13 +328,13 @@ static int masterguard_beeper_status(item_t *item, char *value, const size_t val
 	switch (item->value[0]) {
 		case '0':
 			if (valuelen >= 9)
-				strcpy(value, "disabled");
+				strcpy(value, "disabled");	/* length limit was checked */
 			else
 				*value = '\0';
 			break;
 		case '1':
 			if (valuelen >= 8)
-				strcpy(value, "enabled");
+				strcpy(value, "enabled");	/* length limit was checked */
 			else
 				*value = '\0';
 			break;
@@ -573,7 +573,7 @@ static int masterguard_setvar(item_t *item, char *value, const size_t valuelen)
 			case 's':
 				/* copy to s to avoid snprintf()ing value to itself */
 				if (strlen(value) >= sizeof s) goto ill;
-				strcpy(s, value);
+				strcpy(s, value);	/* length limit was checked */
 				break;
 			default:
 				upsdebugx(2, "setvar: unknown dfl %c", item->dfl[0]);
@@ -767,6 +767,8 @@ static item_t masterguard_qx2nut[] = {
 	 */
 	/* type			flags	rw	command		answer	len	leading	value	from	to	dfl	qxflags	precmd				preans	preproc */
 	{ "battery.charge",	0,	NULL,	"GBS,XX\r",	"",	43,	'(',	"",	4,	6,	"%.0f",	0,	masterguard_add_slaveaddr,	NULL,	NULL },
+	{ "experimental.battery.ageing.factor",	0,	NULL,	"GBS,XX\r",	"",	43,	'(',	"",	18,	21,	"%.0f",	QX_FLAG_NONUT,	masterguard_add_slaveaddr,	NULL,	NULL },
+	{ "experimental.battery.calibration.factor",	0,	NULL,	"GBS,XX\r",	"",	43,	'(',	"",	33,	35,	"%.0f",	QX_FLAG_NONUT,	masterguard_add_slaveaddr,	NULL,	NULL },
 	/* 
 	 * hhhh: hold time (minutes)
 	 * HHHH: recharge time to 90% (minutes)
@@ -864,6 +866,7 @@ static item_t masterguard_qx2nut[] = {
 	/* test.system.start */
 	/* calibrate.start */
 	/* calibrate.stop */
+	{ "clear.fault.record",		0,	NULL,	"FCLR,XX\r",	"",	0,	'\0',	"",	0,	0,	NULL,	QX_FLAG_CMD,	masterguard_add_slaveaddr,	NULL,	NULL },
 	{ "bypass.start",		0,	NULL,	"FOFF\r",	"",	0,	'\0',	"",	0,	0,	NULL,	QX_FLAG_CMD,	NULL,				NULL,	NULL },
 	{ "bypass.stop",		0,	NULL,	"FON\r",	"",	0,	'\0',	"",	0,	0,	NULL,	QX_FLAG_CMD,	NULL,				NULL,	NULL },
 	/* reset.input.minmax */
@@ -1061,6 +1064,8 @@ static void masterguard_makevartable(void) {
 	addvar(VAR_VALUE, "fault_3", "Fault record 3");
 	addvar(VAR_VALUE, "fault_4", "Fault record 4");
 	addvar(VAR_VALUE, "fault_5", "Fault record 5 (oldest)");
+	addvar(VAR_VALUE, "experimental.battery.ageing.factor", "Battery ageing factor (promille, 1000=new)");
+	addvar(VAR_VALUE, "experimental.battery.calibration.factor", "Battery calibration factor (percent)");
 }
 
 
