@@ -56,7 +56,7 @@
 #include "nut_stdint.h"
 
 #define DRIVER_NAME	"Siemens SITOP UPS500 series driver"
-#define DRIVER_VERSION	"0.07"
+#define DRIVER_VERSION	"0.08"
 
 #define RX_BUFFER_SIZE 100
 
@@ -268,6 +268,11 @@ void upsdrv_shutdown(void) {
 void upsdrv_help(void) {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void) {
 	/* allow '-x max_polls_without_data=<some value>' */
