@@ -26,7 +26,9 @@
 #include "powercom-hid.h"
 #include "usb-common.h"
 
-#define POWERCOM_HID_VERSION	"PowerCOM HID 0.71"
+#include <ctype.h>	/* isdigit() */
+
+#define POWERCOM_HID_VERSION	"PowerCOM HID 0.73"
 /* FIXME: experimental flag to be put in upsdrv_info */
 
 /* PowerCOM */
@@ -62,10 +64,30 @@ static char powercom_scratch_buf[32];
  */
 static char powercom_sdcmd_byte_order_fallback = 0;
 
+/* Some devices (Raptor and Smart KING Pro series) follow the protocol
+ * where we can not set arbitrary value of shutdown delay in seconds
+ * (like in other powercom UPSes), but the shutdown delay can be set
+ * only from table "Table of possible delays for Shutdown commands"
+ * specified at page 17 of the protocol document:
+ *   https://networkupstools.org/protocols/powercom/Software_USB_communication_controller_SKP_series.doc
+ *
+ * "Table of possible delays for Shutdown commands" (mentioned for
+ * `DelayBeforeShutdown` and `DelayBeforeStartup` HID Usages):
+ *       Index	1	2	3	4	5	6	7	8	9	10	11	12	13	14	15	16	17	18
+ *       Value	12s	18s	24s	30s	36s	42s	48s	54s	1m	2m	3m	4m	5m	6m	7m	8m	9m	10m
+ * Sent to UPS	.2	.3	.4	.5	.6	.7	.8	.9	01	02	03	04	05	06	07	08	09	10
+ *
+ */
+static char powercom_sdcmd_discrete_delay = 0;
+
 static const char *powercom_startup_fun(double value)
 {
 	uint16_t	i = value;
 
+	/* For powercom_sdcmd_discrete_delay we also read minutes
+	 * from DelayBeforeStartup (same as for older dialects).
+	 * FIXME: ...but theoretically they can be 32-bit (1..99999)
+	 */
 	snprintf(powercom_scratch_buf, sizeof(powercom_scratch_buf), "%d", 60 * (((i & 0x00FF) << 8) + (i >> 8)));
 	upsdebugx(3, "%s: value = %.0f, buf = %s", __func__, value, powercom_scratch_buf);
 
@@ -75,20 +97,40 @@ static const char *powercom_startup_fun(double value)
 static double powercom_startup_nuf(const char *value)
 {
 	const char	*s = dstate_getinfo("ups.delay.start");
-	uint16_t	val, command;
+	uint32_t	val, command;
 	int iv;
 
-	iv = atoi(value ? value : s) / 60;
-	if (iv < 0 || (intmax_t)iv > (intmax_t)UINT16_MAX) {
-		upsdebugx(0, "%s: value = %d is not in uint16_t range", __func__, iv);
+	/* Start with seconds "as is" - convert into whole minutes */
+	iv = atoi(value ? value : s) / 60;	/* minutes */
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) )
+# pragma GCC diagnostic push
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS
+# pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE
+# pragma GCC diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#endif
+	if (iv < 0 || (intmax_t)iv > (intmax_t)UINT32_MAX) {
+#if (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_PUSH_POP) && ( (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TYPE_LIMITS) || (defined HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_TAUTOLOGICAL_CONSTANT_OUT_OF_RANGE_COMPARE) )
+# pragma GCC diagnostic pop
+#endif
+		upsdebugx(0, "%s: value = %d is not in uint32_t range", __func__, iv);
 		return 0;
 	}
 
-	/* COMMENTME: What are we doing here, a byte-swap in the word? */
-	val = (uint16_t)iv;
-	command =  (uint16_t)(val << 8);
-	command += (uint16_t)(val >> 8);
-	upsdebugx(3, "%s: value = %s, command = %04X", __func__, value, command);
+	if (powercom_sdcmd_discrete_delay) {
+		/* Per spec, DelayBeforeStartup reads and writes
+		 * 4-byte "mmmm" values in minutes (1..99999) */
+		command = (uint32_t)iv;
+	} else {
+		/* COMMENTME: What are we doing here, a byte-swap in the word? */
+		val = (uint16_t)iv;
+		command =  (uint16_t)(val << 8);
+		command += (uint16_t)(val >> 8);
+	}
+
+	upsdebugx(3, "%s: value = %s, command = 0x%08X", __func__, value, command);
 
 	return command;
 }
@@ -101,6 +143,12 @@ static const char *powercom_shutdown_fun(double value)
 {
 	uint16_t	i = value;
 
+	/* NOTE: for powercom_sdcmd_discrete_delay mode it seems we
+	 * do not read DelayBeforeShutdown at all (not for time),
+	 * the value is stored and retrieved as DelayBeforeStartup.
+	 * FIXME: Should anything be changed here?
+	 */
+
 	if (powercom_sdcmd_byte_order_fallback) {
 		/* Legacy behavior */
 		snprintf(powercom_scratch_buf, sizeof(powercom_scratch_buf), "%d", 60 * (i & 0x00FF) + (i >> 8));
@@ -120,25 +168,46 @@ static double powercom_shutdown_nuf(const char *value)
 	uint16_t	val, command;
 	int iv;
 
-	iv = atoi(value ? value : s);
+	iv = atoi(value ? value : s);	/* seconds */
 	if (iv < 0 || (intmax_t)iv > (intmax_t)UINT16_MAX) {
 		upsdebugx(0, "%s: value = %d is not in uint16_t range", __func__, iv);
 		return 0;
 	}
 
-	val = (uint16_t)iv;
-	val = val ? val : 1;    /* 0 sets the maximum delay */
-	if (powercom_sdcmd_byte_order_fallback) {
-		/* Legacy behavior */
-		command = ((uint16_t)((val % 60) << 8)) + (uint16_t)(val / 60);
-		command |= 0x4000;	/* AC RESTART NORMAL ENABLE */
+	if (powercom_sdcmd_discrete_delay) {
+		if (iv <= 12)		command = 1;
+		else if (iv <= 18)	command = 2;
+		else if (iv <= 24)	command = 3;
+		else if (iv <= 30)	command = 4;
+		else if (iv <= 36)	command = 5;
+		else if (iv <= 42)	command = 6;
+		else if (iv <= 48)	command = 7;
+		else if (iv <= 54)	command = 8;
+		else if (iv <= 60)	command = 9;
+		else if (iv <= 120)	command = 10;
+		else if (iv <= 180)	command = 11;
+		else if (iv <= 240)	command = 12;
+		else if (iv <= 300)	command = 13;
+		else if (iv <= 360)	command = 14;
+		else if (iv <= 420)	command = 15;
+		else if (iv <= 480)	command = 16;
+		else if (iv <= 540)	command = 17;
+		else				command = 18;
 	} else {
-		/* New default */
-		command = ((uint16_t)((val / 60) << 8)) + (uint16_t)(val % 60);
-		command |= 0x0040;	/* AC RESTART NORMAL ENABLE */
+		val = (uint16_t)iv;
+		val = val ? val : 1;    /* 0 sets the maximum delay */
+		if (powercom_sdcmd_byte_order_fallback) {
+			/* Legacy behavior */
+			command = ((uint16_t)((val % 60) <<  8)) + (uint16_t)(val / 60);
+			command |= 0x4000;	/* AC RESTART NORMAL ENABLE */
+		} else {
+			/* New default */
+			command = ((uint16_t)((val / 60) << 8)) + (uint16_t)(val % 60);
+			command |= 0x0040;	/* AC RESTART NORMAL ENABLE */
+		}
 	}
 
-	upsdebugx(3, "%s: value = %s, command = %04X", __func__, value, command);
+	upsdebugx(3, "%s: value = %s, command = 0x%04X", __func__, value, command);
 
 	return command;
 }
@@ -153,6 +222,7 @@ static double powercom_stayoff_nuf(const char *value)
 	uint16_t	val, command;
 	int iv;
 
+	/* FIXME: Anything for powercom_sdcmd_discrete_delay? */
 	iv = atoi(value ? value : s);
 	if (iv < 0 || (intmax_t)iv > (intmax_t)UINT16_MAX) {
 		upsdebugx(0, "%s: value = %d is not in uint16_t range", __func__, iv);
@@ -171,7 +241,7 @@ static double powercom_stayoff_nuf(const char *value)
 		command |= 0x0080;	/* AC RESTART NORMAL DISABLE */
 	}
 
-	upsdebugx(3, "%s: value = %s, command = %04X", __func__, value, command);
+	upsdebugx(3, "%s: value = %s, command = 0x%04X", __func__, value, command);
 
 	return command;
 }
@@ -314,6 +384,78 @@ static info_lkp_t powercom_overload_conversion[] = {
 	{ 0, NULL, powercom_overload_conversion_fun, NULL }
 };
 
+/* FAIL-SAFE HACK (proper solution welcome) as proposed in
+ *   https://github.com/networkupstools/nut/issues/2766#issuecomment-3751553822 :
+ * Use global udev handle from usbhid-ups.h for a custom conversion function
+ * that reads Report 0xa4 directly via generic USB call. Formally this is
+ * now triggered by a read of ACPresent which is always served, and ends
+ * up requesting another report instead.
+ */
+static const char *powercom_hack_voltage(double value)
+{
+	static char	buf[32];
+	static char	last_valid[32] = "0.0";	/* Cache last valid reading */
+	usb_ctrl_char	data[8];
+	int	ret, retry, max_retries = 1;
+
+	/* We do not care for ACPresent reading received here */
+	NUT_UNUSED_VARIABLE(value);
+
+	/* Global 'udev' is defined in usbhid-ups.h and initialized by main driver */
+	if (!udev)
+		return last_valid;
+
+	/* If we don't have a valid value yet (driver startup), try harder to get one */
+	if (!strcmp(last_valid, "0.0")) {
+		/* Retry up to 5 times (approx 0.5s) */
+		max_retries = 5;
+	}
+
+	for (retry = 0; retry < max_retries; retry++) {
+		/* Portable read of Report 0xa4 (Feature) using NUT's wrapper */
+		/* 0xA1: Dir=IN, Type=Class, Recp=Interface */
+		ret = usb_control_msg(udev, 0xA1, 0x01, (0x03 << 8) | 0xa4, 0, data, sizeof(data), 1000);
+
+		if (ret > 0) {
+			char	msg[32], *start, *end;
+			int	len = (ret > 8) ? 8 : ret;
+			int	j = 0, i;
+
+			/* Skip byte 0 (Report ID) */
+			for (i=1; i<len; i++) {
+				msg[j++] = (char)data[i];
+			}
+			msg[j] = '\0';
+
+			/* Parse " 13.7 2" -> "13.7" */
+			start = msg;
+			while (*start && !isdigit((unsigned char)(*start)) && *start != '.') {
+				start++;
+			}
+			end = start;
+			while (*end && (isdigit((unsigned char)(*end)) || *end == '.')) {
+				end++;
+			}
+			*end = '\0';
+
+			/* VALIDATION */
+			if (strlen(start) > 0 && strchr(start, '.') != NULL) {
+				snprintf(buf, sizeof(buf), "%s", start);
+				snprintf(last_valid, sizeof(last_valid), "%s", start);
+				return buf;
+			}
+		}
+		if (max_retries > 1) {
+			usleep(100000);
+		}
+	}
+	return last_valid;
+}
+
+static info_lkp_t powercom_hack_voltage_lkp[] = {
+	{ 0, NULL, powercom_hack_voltage, NULL }
+};
+
 /* --------------------------------------------------------------- */
 /* Vendor-specific usage table */
 /* --------------------------------------------------------------- */
@@ -397,6 +539,13 @@ static hid_info_t powercom_hid2nut[] = {
  *	{ "battery.voltage.nominal", 0, 0, "UPS.PowerSummary.ConfigVoltage", NULL, "%.0f", HU_FLAG_STATIC, NULL },
  *	{ "battery.voltage.nominal", 0, 0, "UPS.Battery.ConfigVoltage", NULL, "%.0f", HU_FLAG_STATIC, NULL },
  */
+	/* HACK (proper fix welcome): Formally we map battery.voltage
+	 * to *ACPresent* (Report 0x0a) which is always present.
+	 * But then we use our HACK conversion function to ignore the
+	 * passed value, and read and interpret Report 0xa4 instead!
+	 */
+	{ "battery.voltage", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", NULL, "%s", 0, powercom_hack_voltage_lkp },
+
 	{ "battery.charge", 0, 0, "UPS.PowerSummary.RemainingCapacity", NULL, "%.0f", 0, NULL },
 	{ "battery.charge", 0, 0, "UPS.Battery.RemainingCapacity", NULL, "%.0f", 0, NULL },
 	{ "battery.charge.low", 0, 0, "UPS.PowerSummary.RemainingCapacityLimit", NULL, "%.0f", 0, NULL },
@@ -592,6 +741,7 @@ static int powercom_claim(HIDDevice_t *hd)
 
 accept:
 	powercom_sdcmd_byte_order_fallback = testvar("powercom_sdcmd_byte_order_fallback");
+	powercom_sdcmd_discrete_delay = testvar("powercom_sdcmd_discrete_delay");
 
 	return 1;
 }
