@@ -29,7 +29,7 @@
  */
 
 #define DRIVER_NAME	"Generic HID driver"
-#define DRIVER_VERSION	"0.69"
+#define DRIVER_VERSION	"0.71"
 
 #define HU_VAR_WAITBEFORERECONNECT "waitbeforereconnect"
 
@@ -305,6 +305,7 @@ static status_lkp_t status_info[] = {
 	{ "depleted", STATUS(DEPLETED) },
 	{ "timelimitexp", STATUS(TIMELIMITEXP) },
 	{ "fullycharged", STATUS(FULLYCHARGED) },
+	{ "notfullycharged", STATUS(NOTFULLYCHARGED) },
 	{ "awaitingpower", STATUS(AWAITINGPOWER) },
 	{ "fanfail", STATUS(FANFAIL) },
 	{ "nobattery", STATUS(NOBATTERY) },
@@ -411,7 +412,8 @@ info_lkp_t calibration_info[] = {
 	{ 0, "!cal", NULL, NULL },
 	{ 0, NULL, NULL, NULL }
 };
-/* note: this value is reverted (0=set, 1=not set). We report "battery
+/* note: this value is reverted (0=set, 1=not set), often known as
+   "BatteryPresent" in the HID mapping tables. We report "battery
    not installed" rather than "battery installed", so that devices
    that don't implement this variable have a battery by default */
 info_lkp_t nobattery_info[] = {
@@ -464,9 +466,9 @@ info_lkp_t chargerfail_info[] = {
 	{ 0, "!chargerfail", NULL, NULL },
 	{ 0, NULL, NULL, NULL }
 };
-info_lkp_t fullycharged_info[] = { /* used by CyberPower and TrippLite */
+info_lkp_t fullycharged_info[] = { /* used by CyberPower and TrippLite, and some other HIDs */
 	{ 1, "fullycharged", NULL, NULL },
-	{ 0, "!fullycharged", NULL, NULL },
+	{ 0, "notfullycharged", NULL, NULL },
 	{ 0, NULL, NULL, NULL }
 };
 info_lkp_t depleted_info[] = {
@@ -1272,14 +1274,11 @@ void upsdrv_help(void)
 	size_t i;
 	printf("\nAcceptable values for 'subdriver' via -x or ups.conf "
 		"in this driver (exact names here, case-insensitive "
-		"sub-strings may be used, as well as regular expressions): ");
+		"sub-strings may be used, as well as regular expressions):\n");
 
 	for (i = 0; subdriver_list[i] != NULL; i++) {
-		if (i>0)
-			printf(", ");
-		printf("\"%s\"", subdriver_list[i]->name);
+		printf("\t\"%s\"\n", subdriver_list[i]->name);
 	}
-	printf("\n");
 }
 
 /* optionally tweak prognames[] entries */
@@ -1345,6 +1344,9 @@ void upsdrv_makevartable(void)
 	addvar(VAR_FLAG, "powercom_sdcmd_byte_order_fallback",
 		"Set to use legacy byte order for Powercom HID shutdown commands. Either it was wrong forever, or some older devices/firmwares had it the other way around");
 
+	addvar(VAR_FLAG, "powercom_sdcmd_discrete_delay",
+		"Set to use discrete timing table for Powercom HID shutdown commands (Raptor and Smart KING Pro series)");
+
 #if !((defined SHUT_MODE) && SHUT_MODE)
 	addvar(VAR_VALUE, "subdriver", "Explicit USB HID subdriver selection");
 
@@ -1552,6 +1554,85 @@ void upsdrv_initinfo(void)
 
 	dstate_setinfo("driver.version.data", "%s", subdriver->name);
 
+	upsdebugx(1, "%s: Performing an initial UPS data walk with subdriver %s...",
+		__func__, subdriver->name);
+	if (hid_ups_walk(HU_WALKMODE_INIT) == FALSE) {
+		fatalx(EXIT_FAILURE, "Can't initialize data from HID UPS");
+	} else {
+		analyze_mapping_usage();
+	}
+
+	if (!ups_status)
+		upslogx(LOG_WARNING, "%s: No flag bits for 'ups.status' were explicitly reported; "
+			"it is possible a wrong 'subdriver' option was requested or detected "
+			"(in case of problems with device data, consider testing with other "
+			"explicit driver option 'subdriver' values)",
+			__func__);
+
+	upsdebugx(1, "%s: Optionally adjust some threshold values, if applicable and requested to...", __func__);
+
+	/* Set values below from user settings only if supported by UPS */
+	if (dstate_getinfo("battery.charge.low")) {
+		/* Retrieve user defined battery settings */
+		val = getval(HU_VAR_LOWBATT);
+		if (val) {
+			dstate_setinfo("battery.charge.low", "%ld", strtol(val, NULL, 10));
+		}
+	}
+
+	if (dstate_getinfo("ups.delay.start")) {
+		/* Retrieve user defined delay settings */
+		val = getval(HU_VAR_ONDELAY);
+		if (val) {
+			long	l = strtol(val, NULL, 10);
+#if !((defined SHUT_MODE) && SHUT_MODE)
+			if (subdriver == &cps_subdriver
+			 && (l < 60 || l % 60)
+			) {
+				upslogx(LOG_WARNING, "CPS devices tend to round delays by 60 sec down (ondelay=120 is the suggested minimum; see more in the man page)");
+			}
+#endif
+			dstate_setinfo("ups.delay.start", "%ld", l);
+		}
+	}
+
+	if (dstate_getinfo("ups.delay.shutdown")) {
+		/* Retrieve user defined delay settings */
+		val = getval(HU_VAR_OFFDELAY);
+		if (val) {
+			long	l = strtol(val, NULL, 10);
+#if !((defined SHUT_MODE) && SHUT_MODE)
+			if (subdriver == &cps_subdriver
+			 && (l > 0 && (l < 60 || l % 60))
+			) {
+				/* Note: zero and negative values may
+				 * have special meanings for the firmware */
+				upslogx(LOG_WARNING, "CPS devices tend to round delays by 60 sec down (offdelay=60 is the suggested minimum; see more in the man page)");
+			}
+#endif
+			dstate_setinfo("ups.delay.shutdown", "%ld", l);
+		}
+	}
+
+	upsdebugx(1, "%s: Optionally enable instant commands related to shutdown, if applicable...", __func__);
+
+	/* Enable instant commands below only if supported by UPS */
+	if (find_nut_info("load.off.delay")) {
+		/* Adds default with a delay value of '0' (= immediate) */
+		dstate_addcmd("load.off");
+	}
+
+	if (find_nut_info("load.on.delay")) {
+		/* Adds default with a delay value of '0' (= immediate) */
+		dstate_addcmd("load.on");
+	}
+
+	if (find_nut_info("load.off.delay") && find_nut_info("load.on.delay")) {
+		/* Add composite instcmds (require setting multiple HID values) */
+		dstate_addcmd("shutdown.return");
+		dstate_addcmd("shutdown.stayoff");
+	}
+
 	/* init polling frequency for full updates */
 	val = getval(HU_VAR_POLLFREQ);
 	if (val) {
@@ -1854,85 +1935,6 @@ void upsdrv_initups(void)
 		lbrb_log_delay_without_calibrating = 1;
 	}
 
-	upsdebugx(1, "%s: Performing an initial UPS data walk with subdriver %s...",
-		__func__, subdriver->name);
-	if (hid_ups_walk(HU_WALKMODE_INIT) == FALSE) {
-		fatalx(EXIT_FAILURE, "Can't initialize data from HID UPS");
-	} else {
-		analyze_mapping_usage();
-	}
-
-	if (!ups_status)
-		upslogx(LOG_WARNING, "%s: No flag bits for 'ups.status' were explicitly reported; "
-			"it is possible a wrong 'subdriver' option was requested or detected "
-			"(in case of problems with device data, consider testing with other "
-			"explicit driver option 'subdriver' values)",
-			__func__);
-
-	upsdebugx(1, "%s: Optionally adjust some threshold values, if applicable and requested to...", __func__);
-
-	/* Set values below from user settings only if supported by UPS */
-	if (dstate_getinfo("battery.charge.low")) {
-		/* Retrieve user defined battery settings */
-		val = getval(HU_VAR_LOWBATT);
-		if (val) {
-			dstate_setinfo("battery.charge.low", "%ld", strtol(val, NULL, 10));
-		}
-	}
-
-	if (dstate_getinfo("ups.delay.start")) {
-		/* Retrieve user defined delay settings */
-		val = getval(HU_VAR_ONDELAY);
-		if (val) {
-			long	l = strtol(val, NULL, 10);
-#if !((defined SHUT_MODE) && SHUT_MODE)
-			if (subdriver == &cps_subdriver
-			 && (l < 60 || l % 60)
-			) {
-				upslogx(LOG_WARNING, "CPS devices tend to round delays by 60 sec down (ondelay=120 is the suggested minimum; see more in the man page)");
-			}
-#endif
-			dstate_setinfo("ups.delay.start", "%ld", l);
-		}
-	}
-
-	if (dstate_getinfo("ups.delay.shutdown")) {
-		/* Retrieve user defined delay settings */
-		val = getval(HU_VAR_OFFDELAY);
-		if (val) {
-			long	l = strtol(val, NULL, 10);
-#if !((defined SHUT_MODE) && SHUT_MODE)
-			if (subdriver == &cps_subdriver
-			 && (l > 0 && (l < 60 || l % 60))
-			) {
-				/* Note: zero and negative values may
-				 * have special meanings for the firmware */
-				upslogx(LOG_WARNING, "CPS devices tend to round delays by 60 sec down (offdelay=60 is the suggested minimum; see more in the man page)");
-			}
-#endif
-			dstate_setinfo("ups.delay.shutdown", "%ld", l);
-		}
-	}
-
-	upsdebugx(1, "%s: Optionally enable instant commands related to shutdown, if applicable...", __func__);
-
-	/* Enable instant commands below only if supported by UPS */
-	if (find_nut_info("load.off.delay")) {
-		/* Adds default with a delay value of '0' (= immediate) */
-		dstate_addcmd("load.off");
-	}
-
-	if (find_nut_info("load.on.delay")) {
-		/* Adds default with a delay value of '0' (= immediate) */
-		dstate_addcmd("load.on");
-	}
-
-	if (find_nut_info("load.off.delay") && find_nut_info("load.on.delay")) {
-		/* Add composite instcmds (require setting multiple HID values) */
-		dstate_addcmd("shutdown.return");
-		dstate_addcmd("shutdown.stayoff");
-	}
-
 	upsdebugx(1, "%s: finished", __func__);
 }
 
@@ -1994,6 +1996,12 @@ static void process_boolean_info(const char *nutvalue)
 	else if (!strcmp(nutvalue, "offline"))
 		process_boolean_info("!online");
 
+	/* Similarly for (NOT)FULLYCHARGED status that not all devices report */
+	if (!strcmp(nutvalue, "fullycharged"))
+		process_boolean_info("!notfullycharged");
+	else if (!strcmp(nutvalue, "notfullycharged"))
+		process_boolean_info("!fullycharged");
+
 	if (*nutvalue == '!') {
 		nutvalue++;
 		clear = 1;
@@ -2172,6 +2180,7 @@ static bool_t hid_ups_walk(walkmode_t mode)
 	hid_info_t	*item;
 	double		value;
 	int		retcode;
+	int		items_succeeded = 0; /* Track successful polls to detect total failure */
 
 #if !((defined SHUT_MODE) && SHUT_MODE)
 	/* extract the VendorId for further testing */
@@ -2362,13 +2371,19 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			return FALSE;
 
 		case LIBUSB_ERROR_IO:        /* I/O error */
-			/* Uh oh, got to reconnect, with a special suggestion! */
-			dstate_setinfo("driver.state", "reconnect.trying");
+			/* Some devices have firmware bugs causing transient I/O errors
+			 * on specific HID reports. Rather than triggering expensive
+			 * reconnects, skip the failing report and continue. If device
+			 * is truly disconnected, other error codes will catch it, or
+			 * all polls fail and safety check below triggers reconnect. */
+			upsdebugx(3, "Got LIBUSB_ERROR_IO on item '%s' ReportID=0x%02x - skipping",
+				item->info_type ? item->info_type : "(null)",
+				item->hiddata ? item->hiddata->ReportID : 0xFFU);
 			interrupt_pipe_EIO_count++;
-			hd = NULL;
-			return FALSE;
+			continue;
 
 		case 1:
+			items_succeeded++; /* Count successful data retrieval */
 			break;	/* Found! */
 
 		case 0:
@@ -2438,6 +2453,16 @@ static bool_t hid_ups_walk(walkmode_t mode)
 		}
 	}
 
+	/* Safety check: if we got zero successful polls during update,
+	 * device may be truly disconnected (not just transient errors).
+	 * Skip this check during INIT mode where failures are expected. */
+	if (items_succeeded == 0 && (mode == HU_WALKMODE_QUICK_UPDATE || mode == HU_WALKMODE_FULL_UPDATE)) {
+		upsdebugx(1, "Got zero successful data polls - device may be disconnected");
+		dstate_setinfo("driver.state", "reconnect.trying");
+		hd = NULL;
+		return FALSE;
+	}
+
 	return TRUE;
 }
 
@@ -2774,10 +2799,10 @@ static void ups_status_set(void)
 			}
 
 			upslogx(LOG_WARNING, "%s: seems that UPS [%s] is in OL+DISCHRG state now. %s"
-			"Is it calibrating (perhaps you want to set 'onlinedischarge_calibration' option)? "
-			"Note that some UPS models (e.g. CyberPower UT series) emit OL+DISCHRG when "
-			"in fact offline/on-battery (perhaps you want to set 'onlinedischarge_onbattery' option).",
-			__func__, upsname, msg_charge);
+				"Is it calibrating (perhaps you want to set 'onlinedischarge_calibration' option)? "
+				"Note that some UPS models (e.g. CyberPower UT series) emit OL+DISCHRG when "
+				"in fact offline/on-battery (perhaps you want to set 'onlinedischarge_onbattery' option).",
+				__func__, upsname, msg_charge);
 		}
 	} else if ((ups_status & STATUS(ONLINE))) {
 		/* we get here if simply online, not discharging */
@@ -2792,10 +2817,20 @@ static void ups_status_set(void)
 		status_set("DISCHRG");		/* discharging */
 	}
 
-	if ((ups_status & STATUS(CHRG))
-	&& !(ups_status & STATUS(FULLYCHARGED))
-	) {
-		status_set("CHRG");		/* charging */
+	if (ups_status & STATUS(CHRG)) {
+		if (ups_status & STATUS(NOTFULLYCHARGED)) {
+			/* Device reports this status, else below... */
+			status_set("CHRG");		/* charging */
+		} else if (!(ups_status & STATUS(FULLYCHARGED)) && !(ups_status & STATUS(NOTFULLYCHARGED))) {
+			/* Device does not report this status at all */
+			const char	*s;
+			if ((s = dstate_getinfo("battery.charge"))) {
+				/* NOTE: exact "0" may mean a conversion error: */
+				int	current_charge = atoi(s);
+				if (current_charge > 0 && current_charge < 100)
+					status_set("CHRG");		/* charging */
+			}
+		}
 	}
 
 	if (ups_status & (STATUS(LOWBATT) | STATUS(TIMELIMITEXP) | STATUS(SHUTDOWNIMM))) {
